#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Datos exlusivamente de uso interno del demonio del cliente.
"""

import Pyro.core

from common.interfaces.server import IServerDataManager
from common.interfaces.server import IServerIncidenceManager

from common.config import CLIENT_PORT, SERVER_PORT
from config import MOD_FLAG_FILE

SERVER_IP = None

class PyroFactory(object):

    """
    Factoría de objetos remotos PYRO.

    Esta clase es utilizada para obtener desde cualquier parte del código del
    cliente, los objetos remotos del servidor bien configurados. Se debe hacer
    uso de esta factoría en vez de pasar objetos remotos (proxies en realidad)
    por parámetro.
    """

    server_ip = None

    def set_server_ip (server_ip):

        ''' Fijar una dirección IP para el servidor I{Vaguada}.

            Este método existe únicamente para poder lanzar clientes especificando
            su código y la IP del servidor dinámicamente, en la línea de órdenes.

            @param server_ip: dirección IP o nombre de un servidor I{Vaguada}.
                              Ejemplo: C{'vaguada-server'}.
            @type server_ip: L{str} '''

        PyroFactory.server_ip = server_ip
    set_server_ip = staticmethod (set_server_ip)


    def get_server_data_manager ():
        """Conecta con el servidor y obtiene un proxy a su DataManager

        @rtype: L{IServerDataManager}
        @return: Objeto remoto L{IServerDataManager} del servidor. 
        """

        proxy = Pyro.core.getProxyForURI("PYROLOC://%s:%s/IServerDataManager" %\
                (PyroFactory.server_ip, SERVER_PORT))

        return proxy
    get_server_data_manager = staticmethod(get_server_data_manager)


    def get_server_incidence_manager ():
        """Conecta con el servidor y obtiene un proxy a su IncidenceManager

        @rtype: L{IServerIncidenceManager}
        @return: Objeto remoto L{IServerIncidenceManager} del servidor. 
        """

        proxy = Pyro.core.getProxyForURI("PYROLOC://%s:%s/IServerIncidenceManager" %\
                (PyroFactory.server_ip, SERVER_PORT))

        return proxy
    get_server_incidence_manager = staticmethod(get_server_incidence_manager)


    def get_ui_assistant(server_ip = None):
        """ Conecta con el IUIAssistant del cliente y obtiene un proxy.

        @rtype: L{IUIAssistant}
        @return: Objeto remoto L{UIAssistant} del cliente.
        """
        uri = "PYROLOC://localhost:%s/IUIAssistant" % CLIENT_PORT
        proxy = Pyro.core.getProxyForURI(uri) 
        return proxy
    get_ui_assistant = staticmethod(get_ui_assistant)
