#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Implementaciones de las interfaces relativas al cliente.

Cada clase implementada aquí será un objeto PYRO.
"""

import time
import logging
import os

import Pyro.core

from common.interfaces.client import IClientDataManager
from common.interfaces.client import IClientIncidenceManager
from common.interfaces.server import IServerDataManager
from common.interfaces.client import IClientUIAssistant
from common.incidences import IncidenceData
from common.data import FileMetaDataList
from common.data import FileData
from data import PyroFactory
from utils import connection_safe
from utils import GuadalinfoId


class ClientDataManager(IClientDataManager, Pyro.core.ObjBase):
    """
    Implementación PYRO de la clase L{IClientDataManager}.
    """
    def __init__(self):

        Pyro.core.ObjBase.__init__(self)
        self.centre_code = DataManager.get_instance().centre_code
        self.logger = logging.getLogger()

    def ping(self):
        return True


    def get_datetime(self):
        return time.localtime()


    def set_code(self, new_code):
        gid = GuadalinfoId()
        gid['ID_PROVINCIA'] = str(new_code[0])
        gid['ID_LOCALIDAD'] = str(new_code[1])
        DataManager.get_instance().centre_code = new_code


    def set_dispatch_list(self, dispatch_list):

        # Prepare local list
        # FIXME Para depurar, se crea un FileMetaDataList con 5 elementos.
        self.logger.debug("dispatch_list received from server: %s" %\
                str(dispatch_list.sumUp()))

        local_list = FileMetaDataList(debug = [1, 2, 3])
        
        # Calculate diff
        missed_in_local = local_list.missed(dispatch_list)
        only_in_local = dispatch_list.missed(local_list)

        self.logger.debug("missed_in_local: %s" % str(missed_in_local.sumUp()))
        self.logger.debug("only_in_local: %s" % str(only_in_local.sumUp()))

        # Send files in different thread.
        self.__set_files(only_in_local)
        self.__get_files(missed_in_local)


        # FIXME Devolver algo coherente con la definición de la interfaz
        return (only_in_local, missed_in_local)

    def set_ssh(self, value):
        if value:
            os.system('/usr/share/vaguada/open_port')
        else:
            os.system('/etc/init.d/rc.firewall start')




class ClientIncidenceManager(IClientIncidenceManager, Pyro.core.ObjBase):
    """
    Implementación PYRO de la clase L{IClientIncidenceManager}.
    """

    def __init__(self):
        Pyro.core.ObjBase.__init__(self)
        self.__incidence_manager = IncidenceManager.get_instance()


    def get_incidences(self):
        return self.__incidence_manager.get_incidences_data()


    def get_ninc(self):
        return len(self.__incidence_manager.incidences)


    def is_active(self, incidence_data):
        inc_indents = self.__incidence_manager.incidences.get_keys()
        return incidence_data.ident in inc_idents


    def solve(self, incidence_data, method_pos, *params):

        return self.__incidence_manager.solve(incidence_data, method_pos, *params)


class ClientUIAssistant(IClientUIAssistant, Pyro.core.ObjBase):
    """
    Implementación PYRO de la clase L{IUIAssistant}.
    """
    def __init__(self):
        Pyro.core.ObjBase.__init__(self)

        self.__incidence_manager = IncidenceManager.get_instance()
 

    def get_incidences(self, centre_code = None):
        all_incidences = self.__incidence_manager.get_incidences_data()

        for incidence in all_incidences:
            incidence.methods = []

        return all_incidences


    def solve_incidence(self, incidence, method_pos, *params):
        return None


    def get_inc_log(self, centre_code = None, lines = None):
        sdm = PyroFactory.get_server_data_manager()
        centre_code = DataManager.get_instance().centre_code

        return sdm.get_inc_log(centre_code, lines)

    get_inc_log = connection_safe(get_inc_log)

    
    def get_sync_log(self, centre_code = None, lines = None):
        sdm = PyroFactory.get_server_data_manager()
        centre_code = DataManager.get_instance().centre_code

        return sdm.get_sync_log(centre_code, lines)

    get_sync_log = connection_safe(get_sync_log)




from managers import IncidenceManager
from managers import DataManager
