#!/usr/bin/env python
# -*- coding: utf-8 -*-

import crypt
import ldap
import string
import re
import sys
import datetime
import time
import os
import os.path
import logging
import md5

from client.daemon.config import DATA_DIR, CHECKSUM_FILE
from common.data import ChecksumCache
from common.utils import get_last_file_date

def parse_ldap():
    content = get_ldap_content()

    now = datetime.date.today().toordinal()
    now = datetime.date.fromordinal(now - 1)
    (last, last_content) = get_last_file_date(DATA_DIR + os.sep + 'data', '.users')

    if last:
        diff_in_days = (now - last).days

        if diff_in_days >= 2:
            for did in range(1, diff_in_days ):
                this_date = datetime.date.fromordinal(now.toordinal() - did)

                aux = str(this_date).split('-')
                this_date_str = aux[0] + aux[1] + aux[2]
                file_ident = 'data' + os.sep + this_date_str + '.users'
                filepath = DATA_DIR + os.sep + file_ident
                fileobj = open(filepath, 'w')
                fileobj.write(last_content)
                fileobj.close()

                checksum_cache = ChecksumCache(CHECKSUM_FILE)
                md5sum = md5.new(last_content)
                checksum_cache[file_ident] = md5sum.hexdigest()

    aux = str(now).split('-')
    date_str = aux[0] + aux[1] + aux[2]
    file_ident = 'data' + os.sep + date_str + '.users'

    if not os.path.exists(file_ident):
        filepath = DATA_DIR + os.sep + file_ident
        fileobj = open(filepath, 'w')
        fileobj.write(content)
        fileobj.close()
        checksum_cache = ChecksumCache(CHECKSUM_FILE)
        md5sum = md5.new(content)
        checksum_cache[file_ident] = md5sum.hexdigest()



def parse_date(date):
	list = string.split(date, "/")
	list.reverse()
	return int(string.join(list, ""))


def get_ldap_content():
    last_date = 0
    
    # Get 'localidad' and 'provincia' from config file
    def read_conf(file):
    	f = open(file, "r")
    	f_content = f.read()
    # Cambiado para que funcione con ids no numricas
    #	result = re.findall('ID_LOCALIDAD="(\d+)"', f_content)
    	result = re.findall('ID_LOCALIDAD="(\S+)"', f_content)
    	localidad = result[0]
    #	result = re.findall('ID_PROVINCIA="(\d+)"', f_content)
    	result = re.findall('ID_PROVINCIA="(\S+)"', f_content)
    	provincia = result[0]
    	return (localidad, provincia)
    
    now = datetime.date.today()
    
    (localidad, provincia) = read_conf("/etc/guadalinfo_id.conf")
    
    conn = ldap.initialize("ldap://localhost")
    
    attr_list = ['uid', 'sex', 'education', 'birthDate', 'workActivity', 'income', 'startDate']
    conn.simple_bind('cn=admin, o=guadalinex, c=es', 'wadA15')
    query_result = conn.search('ou=People,o=guadalinex,c=es', ldap.SCOPE_SUBTREE, '(uid=*)', attr_list)
    
    content = ""
    
    while 1:
    	result_type, result_data = conn.result(query_result, 0)
    	if (result_data == []):
    		break
    	else:
    		if result_type == ldap.RES_SEARCH_ENTRY:
    			(dn, attrs) = result_data[0]
    			try:
    				user = crypt.crypt(attrs['uid'][0], "dc")
    				for i in attr_list:
    					if not attrs.has_key(i):
    						attrs[i] = ['']
    				item = user + ";" + attrs['sex'][0] + ";" + attrs['education'][0] + ";" + attrs['birthDate'][0] + ";" + attrs['workActivity'][0] + ";" + attrs['income'][0] + ";" + attrs['startDate'][0] + ";" + now.strftime("%m")
    				content = content + item + "\n"
    				if attrs['startDate'][0] != '':
    					date = parse_date(attrs['startDate'][0])
    					if date > last_date:
    						last_date = date
    			except TypeError, msg:
    				raise
    			except:
    				logging.getLogger().error("Error: %s" % sys.exc_info()[0])
    
    #try:
    #	f_dest = open("/tmp/" + provincia + "_" + localidad + "_" + str(last_date) + "_usuarios", "w")
    #except:
    #	f_dest = open("/tmp/" + provincia + "_" + localidad + "_" + "0" + "_usuarios", "w")
    #f_dest.write(content)
    #f_dest.close()
    conn.unbind()
    return content

def _get_last_file_date(path, extension = '.users'):
   
    file_list = []
    maxdate = '0'
    for filepath in os.listdir(path):
        if filepath.endswith(extension):
            file_date = filepath.split('.')[0]
            if file_date > maxdate:
                maxdate = file_date

    if maxdate != '0':
        max_content = open(path + os.sep + 'data' + os.sep + maxdate + extension).read()
        try:
            date = datetime.date(int(maxdate[:4]), int(maxdate[4:6]), int(maxdate[6:8]))
            return (date, max_content)
        except:
            return (None, None)
    else:
        return (None, None)



