#!/usr/bin/env python
# -*- coding: utf-8 -*-

import string
import time
import re
import crypt
import datetime
import os
import os.path
import md5
import gzip
import logging

from sets import Set

from client.daemon.data import PyroFactory
from common.data import ModCache
from common.data import ChecksumCache
from client.daemon.config import DATA_DIR, CHECKSUM_FILE, MOD_FLAG_FILE

from common.utils import get_last_file_date

months = {'jan': '01', 'feb': '02', 'mar': '03', 'apr': '04', 'may': '05', 'jun': '06', 'jul': '07', 'aug': '08', 'sep': '09', 'oct': '10', 'nov': '11', 'dec': '12', 'ene': '01', 'abr': '04', 'ago': '08', 'dic': '12'}

# Get 'localidad' and 'provincia' from config file
def read_conf(file):
    f = open(file, "r")
    f_content = f.read()
# Cambiado para que funcione con ids no numéricas
#   result = re.findall('ID_LOCALIDAD="(\d+)"', f_content)
    result = re.findall('ID_LOCALIDAD="(\S+)"', f_content)
    localidad = result[0]
#   result = re.findall('ID_PROVINCIA="(\d+)"', f_content)
    result = re.findall('ID_PROVINCIA="(\S+)"', f_content)
    provincia = result[0]
    return (localidad, provincia)

# Read lines from log file and return logins and logout as event objects
class AuthParser:
    
    def __init__(self, file):
        if file.endswith('.gz'):
            self.source = gzip.open(file, "r")
        else:
            self.source = open(file, "r")

        passwd = open("/etc/passwd", "r")
        self.passwd_content = passwd.read()
        passwd.close()


    def in_passwd(self, user):
        try:
            return re.search(user, self.passwd_content)
        except:
            logger = logging.getLogger() 
            logger.warning('Error buscando en /etc/passwd el usuario: %s' % user)


    def parse_line(self):
        line = self.source.readline()
        while line:
            
            if re.search('(opened)', line):
                ## Apr 13 07:53:01 seraphim cron(pam_unix)[9909]: session opened for user mail by (uid=0)
                #pattern = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\((\S+)\)\[(\d+)\]: session opened for user (\S+)'
                #type = "opened"
                # Almacena todos los patrones que se van a comparar
                patterns = []

                # Apr 13 07:53:01 seraphim cron(pam_unix)[9909]: session opened for user mail by (uid=0)
                pattern1 = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\((\S+)\)\[(\d+)\]: session opened for user (\S+)'

                # May  3 07:56:54 servidor gdm[1356]: (pam_unix) session opened for user usuario by (uid=0)
                pattern2 = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\[(\d+)\]: \((\S+)\) session opened for user (\S+)'

                # Almacenamos todos los patrones
                patterns.append({'index': 1, 'content': pattern1})
                patterns.append({'index': 2, 'content': pattern2})

                type = "opened"

            elif re.search('(closed)', line):
                ## Apr 13 07:53:01 seraphim cron(pam_unix)[9909]: session closed for user mail
                #pattern = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\((\S+)\)\[(\d+)\]: session closed for user (\S+)'
                #type = "closed"
                patterns = []

                # Apr 13 07:53:01 seraphim cron(pam_unix)[9909]: session closed for user mail
                pattern1 = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\((\S+)\)\[(\d+)\]: session closed for user (\S+)'

                # May  3 07:56:54 servidor gdm[1356]: (pam_unix) session closed for user usuario by (uid=0)
                pattern2 = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\[(\d+)\]: \((\S+)\) session closed for user (\S+)'

                patterns.append({'index': 1, 'content': pattern1})
                patterns.append({'index': 2, 'content': pattern2})

                type = "closed"
            else:
                line = self.source.readline()
                continue
            # result[0][0]: month, result[0][1]: day, result[0][2]: hours, result[0][3]: minutes, result[0][4]: seconds,
            # result[0][5]: source, result[0][6]: n/a, result[0][7]: n/a, result[0][8]: id, result[0][9]: user
            #result = re.findall(pattern, line);
            for patt in patterns:
                result = re.findall(patt['content'], line)
                if result:
                    break

            if result and (result[0][9] == "usuario" or result[0][9] == "root"\
                    or result[0][9] == "dinamizador" or not self.in_passwd(result[0][9])):
                user = crypt.crypt(result[0][9], "dc")
                #debug
                #if type == "opened":
                #    AuthParser.num_open += 1
                #elif type == "closed":
                #    AuthParser.num_close +=1
                #/debug
                if patt['index'] == 1:
                    data = (result[0][0], result[0][1], result[0][2],
                            result[0][3], result[0][4], result[0][5],
                            result[0][8], user)
                elif patt['index'] == 2:
                    data = (result[0][0], result[0][1], result[0][2],
                            result[0][3], result[0][4], result[0][5],
                            result[0][7], user)
                #data = (result[0][0], result[0][1], result[0][2], result[0][3], result[0][4], result[0][5], result[0][8], user)
                item = Event(type, data)
                return item
            line = self.source.readline()

# Holds one login or logout
class Event:

    def __init__(self, type = "any", data = ['?', '?', '?', '?', '?', '?', '?', '?']):
        self.type = type
        self.month = data[0]

        self.day = data[1]

        if len(self.day) == 1:
            self.day = '0' + self.day

        self.hours = data[2]
        self.minutes = data[3]
        self.seconds = data[4]
        self.source = data[5]
        self.id = data[6]
        self.user = data[7]
        self.year = self.__get_year()


    def __get_year(self):
        if self.month == '?':
            return '?'

        result = None
        (server_year, server_month, server_day) = time.gmtime(server_time)[:3]

        server_date = datetime.date(server_year, server_month, server_day)

        local_month = int(months[self.month.lower()])
        local_day = int(self.day)
        local_date = datetime.date(server_year, local_month, local_day)

        if server_date < local_date:
            result = server_year - 1
        else:
            result = server_year

        return str(result)



# Holds one Sessions object per source (ip or hostname)
class Report:

    def __init__(self):
        self.source_list = {}


    def in_report(self, source_id):
        return self.source_list.has_key(source_id)

    
    def add(self, event):
        nmonth = months[event.month.lower()]
        filename = ''.join([event.year, nmonth, event.day, '.sessions'])

        today = datetime.date.today()
        event_date = datetime.date(int(event.year), 
                int(nmonth), 
                int(event.day))

        if today > event_date:
            if not self.in_report(filename):
                self.source_list[filename] = Sessions()

            self.source_list[filename].add(event)


    def display(self, sep):
        result = ""
        for source in self.source_list:
            sessions = self.source_list[source]
            result = result + sessions.display(source, sep)
        return result



# Holds a list of login events, a list of logout events and a list to prevent multiple event ids
class Sessions:
    num_reps = 0
    num_root = 0
    def __init__(self):
        self.opened = {}
        self.closed = {}
        self.id_table = {}


    def add(self, event):
        if event.type == "opened":
            #new_id = int(event.id)
            #new key contains pid, user and client
            key = (event.id, event.user, event.source, event.month, event.day, 0)
            new_key = key
            # Finds new id (previously unallocated) if needed
            while self.opened.has_key(new_key):
                new_key = (key[0], key[1], key[2], key[3], key[4], new_key [5] + 1)
            if self.opened.has_key(new_key):
                Sessions.num_reps += 1
            self.opened[new_key] = event
            #print string.join((event.hours, event.minutes, event.seconds),":"), event.user, event.source, event.id, event.type, event.month
            actual=self.opened[key]
            #print "+",key[0], key[1], key[2], key[3], key[4], key [5], string.join((actual.hours, actual.minutes, actual.seconds),":")
            #print "-",event.id, event.user, event.source, event.month, event.day, new_key[5], string.join((event.hours, event.minutes, event.seconds),":")
            # If new id, stores it in id_table
            if key != new_key:
                self.id_table[key] = new_key
        elif event.type == "closed":
            key = (event.id, event.user, event.source, event.month, event.day, 0)
            new_key=key
            if self.id_table.has_key(key):
                new_key = self.id_table[key]
            self.closed[key] = event
        else:
            return
        last_event.month = event.month
        last_event.day = event.day

    def get_sessions(self, sep = ';'):
        result = []
        for event_id in self.opened:
            open_event = self.opened[event_id]
            line = string.join((provincia, localidad, open_event.source, open_event.user, open_event.month, open_event.day, string.join((open_event.hours, open_event.minutes, open_event.seconds), ":")), sep)
            if self.closed.has_key(event_id):
                close_event = self.closed[event_id]
                line += sep + string.join((close_event.month, close_event.day, string.join((close_event.hours, close_event.minutes, close_event.seconds), ":")), sep) + sep + now.strftime("%m")
            else:
                line += sep + "?" + sep + "?" + sep + "?" + sep + now.strftime("%m")

            result.append(line)
        return result


    def display(self, source, sep):
        result = ""
        for event_id in self.opened:
            open_event = self.opened[event_id]
            result = result + string.join((provincia, localidad, source, open_event.user, open_event.month, open_event.day, string.join((open_event.hours, open_event.minutes, open_event.seconds), ":")), sep)
            if self.closed.has_key(event_id):
                close_event = self.closed[event_id]
                result = result + sep + string.join((close_event.month, close_event.day, string.join((close_event.hours, close_event.minutes, close_event.seconds), ":")), sep) + sep + now.strftime("%m")
            else:
                result = result + sep + "?" + sep + "?" + sep + "?" + sep + now.strftime("%m")
            result = result + "\n"
        return result


def get_first_in_sessions(keys):
    mindate = min(keys)
    try:
        date = datetime.date(int(mindate[:4]), int(mindate[4:6]), int(mindate[6:8]))
        return date
    except:
        return None


def calculate_minimal_date(session_dates):
    """ Calcula la fecha menor de entre la más pequeña de las fechas de las
    sesiones a guardar (session_dates) y la mayor de las fechas de los ficheros
    de sesiones en disco
    """
    # Calculate minimal date
    last_in_disk = get_last_file_date(DATA_DIR + os.sep + 'data', '.sessions')[0]
    first_in_sessions = get_first_in_sessions(session_dates)

    if last_in_disk and first_in_sessions:
        min_ordinal = min(last_in_disk.toordinal(), first_in_sessions.toordinal())
        min_date = datetime.date.fromordinal(min_ordinal)
    elif last_in_disk:
        min_date = last_in_disk
    elif first_in_sessions:
        min_date = first_in_sessions
    else:
        min_date = None
    return min_date

def generate_empty_files(min_date):
    """ Genera un fichero de sesiones vacío (siempre y cuando no se sobreescriba algún
    fichero) para los dias comprendidos entre min_date y la fecha actual.
    """
    now = datetime.date.today()

    for day in range(min_date.toordinal() + 1 , now.toordinal()):
        this_day = datetime.date.fromordinal(day)
        aux = str(this_day).split('-')
        date_str = aux[0] + aux[1] + aux[2]
        filepath = DATA_DIR + os.sep + 'data' + os.sep + date_str + '.sessions'
        if not os.path.exists(filepath):
            open(filepath, 'w').close()
            checksum_cache = ChecksumCache(CHECKSUM_FILE)
            md5sum = md5.new(open(filepath,'r').read())
            file_ident = 'data' + os.sep + date_str + '.sessions'
            checksum_cache[file_ident] = md5sum.hexdigest()


def generate_files(session_dict):
    """ Genera los ficheros de session a partir de un diccionario con la
    información de las sesiones

    @type session_dict: dict
    @param session_dict: Diccionario cuyas claves son las fechas de las sesiones
    y cuyos valores son listas de valores de las sesiones en sí.
    """
    min_date = calculate_minimal_date(session_dict.keys())

    modcache = ModCache(MOD_FLAG_FILE)

    for filename in session_dict.keys():
        filepath = DATA_DIR + os.sep + 'data' + os.sep + filename
        directory = filepath.split(os.sep)[-2]
        file_ident = directory + os.sep + filename

        diff_set = None
        # Si el fichero existe, añadimos la diferencia entre los conjutos de
        # sesiones.
        if os.path.exists(filepath):
            # Si un fichero existe, y se encuentra sesiones de ese día, se toman
            # dichas sesiones como inválidas.
            continue

            #content = [ele.strip() for ele in open(filepath).readlines()]

            #content_set = Set(content)
            #sessions_set = Set(session_dict[filename].get_sessions())
            #diff_set = sessions_set - content_set 
            #if diff_set:
            #    modcache.add(file_ident)
        else:
            diff_set = session_dict[filename].get_sessions()

        if diff_set:
            fileobj = open(filepath, 'a')
            for session in diff_set:
                fileobj.write(session + '\n')
            fileobj.close()
            checksum_cache = ChecksumCache(CHECKSUM_FILE)
            md5sum = md5.new(open(filepath,'r').read())
            checksum_cache[file_ident] = md5sum.hexdigest()
            modcache.add(file_ident)


    if min_date:
        generate_empty_files(min_date)


def parse_auth_log():
    global now
    global last_event
    global provincia
    global localidad
    global server_time

    sdm = PyroFactory.get_server_data_manager()
    
    try:
       server_time = sdm.get_time()
    except:
       return None

    server_time=time.time()
    
    try:
       server_time = sdm.get_time()
    except:
       return None

    now = datetime.date.today()
    last_event = Event()
    (localidad, provincia) = read_conf("/etc/guadalinfo_id.conf")

    result = {}

    files_path = [ele for ele in os.listdir('/var/log/') if\
                  ele.startswith('auth.')]
    num_events = 0
    for file_path in files_path:
        parser = AuthParser("/var/log/" + file_path)
        guadalinfo = Report()
      
        event = parser.parse_line()
        while (event):
            guadalinfo.add(event)
            event = parser.parse_line()
        
        date = datetime.date.today()
        
        split_date = string.split(str(date), "-")

        if guadalinfo.source_list:
            generate_files(guadalinfo.source_list)
            result.update(guadalinfo.source_list)


    return result
    

if __name__ == '__main__':
    parse_auth_log()
