#!/usr/bin/python
# -*- coding: utf-8 -*-

""" Script de pruebas de las cases definidas en el módulo L{incidences}.
"""

from client.daemon.incidences import DummyIncidence
from client.daemon.managers import IncidenceManager
import unittest



class TestDummyIncidence(unittest.TestCase):

    def setUp(self):
        self.incidence = DummyIncidence()


    def test_solve(self):
        """ Probando método de resolución correcto.
        """
        self.assertEqual(self.incidence.solve(1), True)

    def test_no_solve(self):
        """ Probando método de resolución fallido.
        """
        self.assertEqual(self.incidence.solve(2), False)

    def test_methods_order(self):
        """ Probando que el orden de los métodos se respeta.
        """
        methods = self.incidence.methods

        self.assertEqual(methods, 
                ['Manual', 'Resolver','No Resolver'])


class TestIncidenceManager(unittest.TestCase):

    def setUp(self):
       self.im = IncidenceManager.get_instance()


    def test_new_incidence(self):
        """ Probando a añadir una nueva incidencia
        """
        im = self.im
        old_ninc = len(im.incidences)
        incidence = DummyIncidence()

        IncidenceManager.get_instance().new_incidence(incidence)
        new_ninc = len(im.incidences)

        self.assertEqual(old_ninc + 1, new_ninc)


    def test_solve(self):
        """ Probando método de resolución correcto.
        """
        incidence = DummyIncidence()
        self.im.new_incidence(incidence)

        self.assertEqual(self.im.solve(incidence, 1), True)


    def test_no_solve(self):
        """ Probando método de resolución fallido. 
        """
        incidence = DummyIncidence()
        self.im.new_incidence(incidence)

        self.assertEqual(self.im.solve(incidence, 2), False)


    def test_no_method(self):
        """ Probando a resolver una incidencia con un método que no existe. 

        Se debe devolver False.
        """
        incidence = DummyIncidence()
        self.im.new_incidence(incidence)

        self.assertEqual(self.im.solve(incidence, 100), False)


    def test_no_incidence(self):
        """ Probando a resolver una incidencia que no existe.

        Se debe devolver True, ya que si la incidencia no existe, probablemente
        es porque ya se ha resuelto.
        """
        incidence = DummyIncidence()
        inc2 = DummyIncidence()
        self.im.new_incidence(incidence)

        self.assertEqual(self.im.solve(inc2, 0), True)


if __name__ == '__main__':
    unittest.main()

